/*
 * Copyright (c) 2004, Bull SA. All rights reserved.
 * Created by:  Laurent.Vivier@bull.net
 * This file is licensed under the GPL license.  For the full content
 * of this license, see the COPYING file at the top level of this
 * source tree.
 */

/*
 * assertion:
 *
 * aio_lio_opcode shall be ignored.
 *
 * method:
 *
 *      - write data to a file
 *      - fill in an aiocb with an LIO_WRITE aio_lio_opcode
 *      - call aio_read with this aiocb
 */

#define _XOPEN_SOURCE 600
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <aio.h>
#define BUF_SIZE 111

int main()
{
        char tmpfname[256];
        unsigned char buf[BUF_SIZE];
        unsigned char check[BUF_SIZE];
        int fd;
        struct aiocb aiocb1;
        int i;


        snprintf(tmpfname, sizeof(tmpfname), "tmpfile_%d", getpid());
        unlink(tmpfname);
        fd = open(tmpfname, O_CREAT | O_RDWR | O_EXCL,
                  S_IRUSR | S_IWUSR);
        if (fd == -1)
        {
                printf(" Error at open(): %s\n",
                       strerror(errno));
                exit(1);
        }

        unlink(tmpfname);

        for (i = 0; i < BUF_SIZE; i++)
                buf[i] = i;

        if (write(fd, buf, BUF_SIZE) != BUF_SIZE)
        {
                printf(" Error at write(): %s\n",
                       strerror(errno));
                exit(1);
        }

        memset(check, 0xaa, BUF_SIZE);
        memset(&aiocb1, 0, sizeof(struct aiocb));
        aiocb1.aio_fildes = fd;
        aiocb1.aio_buf = check;
        aiocb1.aio_nbytes = BUF_SIZE;

        if (aio_read(&aiocb1) == -1)
        {
                printf(" Error at aio_read(): %s\n",
                       strerror(errno));
                exit(2);
        }

        int err;
        int ret;

        /* Wait until end of transaction */
        while ((err = aio_error (&aiocb1)) == EINPROGRESS);

        err = aio_error(&aiocb1);
        ret = aio_return(&aiocb1);

        if (err != 0)
        {
                printf(" Error at aio_error() : %s\n", strerror (err));
                close(fd);
                exit(2);
        }

        if (ret != BUF_SIZE)
        {
                printf(" Error at aio_return()\n");
                close(fd);
                exit(2);
        }

        /* check it */

        for (i = 0; i < BUF_SIZE; i++)
        {
                if (buf[i] != check[i])
                {
                        printf(" read values are corrupted\n");
                        exit(2);
                }
        }

        close(fd);
        printf ("Test PASSED\n");
        return 0;
}

